@@ -7,8 +7,8 @@ sub new {
         my $class = shift;
         my $self = {};
         
-        my $message = shift;
-        print STDERR "$message\n";
+        my $args = shift;
+        print STDERR "$args->{name}\n";
         
         return bless $self, $class;
 }
